#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "df.h"
#include "mfsd.h"
#include "inspectHDF.h"


typedef union {
    char8 char_8;
    uchar8 uchar_8;
    int8 int_8;
    uint8 uint_8;
    int16 int_16;
    uint16 uint_16;
    int32 int_32;
    uint32 uint_32;
    float32 float_32;
    float64 float_64;
} all_type;		/* used for min/max values */


/*
 * Function: infoSDS.
 *
 * Purpose: Get Info on Scientific Data Sets, if they exists.
 *
 */

void infoSDS(hdf_file, settings)
 char *hdf_file;
 settype *settings;
{
  int i, j, datasets, rank;
  uint16 reference;
  int32 dimsizes[MAXRANK], datatype;
  char *dt;

  all_type min, max;

  char label[MAXLEN], unit[MAXLEN], format[MAXLEN], coordsys[MAXLEN];

  if ((datasets = DFSDnumber(hdf_file)) != FAIL)
    for (i = 0; i < datasets; i++)
      {
       /*
        * * Retreive Info in SDS.
        * *  REQUIRED:  rank, dimensions, reference number.
        * *  OPTIONAL:  label, units, format, coordinate system, max, min,
        * *             and annotations.
        */

	if (DFSDgetdims(hdf_file, &rank, dimsizes, MAXRANK) == FAIL)
	  error(" Error at DFSDgetdims.");

	reference = DFSDlastref();

	printf("\n*Scientific Data Set #%3d  (reference #%d)\n", i, reference);
	printf(" ------------------------\n\n");

	if (DFSDgetdatastrs(label, unit, format, coordsys) != FAIL)
	  {
	    printf("  **Label : \"%s\"\n", label);
	    printf("  **Units : %s\n", unit);
	    printf("  **Format: %s\n", format);
	    printf("  **Coord system: %s\n", coordsys);
	  }
	printf("  **Rank: %d\n", rank);

       /*
        * * Print dimensions in a column layout, with units listed in parenthsis
        * *  (if they exists).
        */

	printf("  **Dimensions:\n");
	for (j = 0; j < rank - 1; j++)
	  {
	    DFSDgetdimstrs(j + 1, label, unit, format);
	    printf("               %d %s", dimsizes[j], label);
	    if (strcmp(unit, "") != 0)
	      printf("   (%s) x\n", unit);
	    else
	      printf("   x\n");
	  }
	DFSDgetdimstrs(rank, label, unit, format);
	printf("               %d %s", dimsizes[rank - 1], label);
	if (strcmp(unit, "") != 0)
	  printf("   (%s)\n\n", unit);
	else
	  printf("\n\n");
       /*
        * * get and display the data type used in the SDS.
        */

	if (DFSDgetNT(&datatype) == FAIL)
	  error(" Error in getting data type.");

	switch (datatype)
	  {
	    case DFNT_FLOAT32:
	      dt = "FLOAT32";
	      if (DFSDgetrange((void *) &(max.float_32), (void *) &(min.float_32))
		  != FAIL)
		printf("  **max: %0.0f  min: %0.0f\n\n",
		       max.float_32, min.float_32);
	      break;
	    case DFNT_FLOAT64:
	      dt = "FLOAT64";
	      if (DFSDgetrange((void *) &(max.float_64), (void *) &(min.float_64))
		  != FAIL)
		printf("  **max: %0.0f  min: %0.0f\n\n",
		       max.float_64, min.float_64);
	      break;
	    case DFNT_INT8:
	      dt = "INT8";
	      if (DFSDgetrange((void *) &(max.int_8), (void *) &(min.int_8))
		  != FAIL)
		printf("  **max: %d  min: %d\n\n",
		       max.int_8, min.int_8);
	      break;
	    case DFNT_UINT8:
	      dt = "UINT8";
	      if (DFSDgetrange((void *) &(max.uint_8), (void *) &(min.uint_8))
		  != FAIL)
		printf("  **max: %d  min: %d\n\n",
		       max.uint_8, min.uint_8);
	      break;
	    case DFNT_INT16:
	      dt = "INT16";
	      if (DFSDgetrange((void *) &(max.int_16), (void *) &(min.int_16))
		  != FAIL)
		printf("  **max: %d  min: %d\n\n",
		       max.int_16, min.int_16);
	      break;
	    case DFNT_UINT16:
	      dt = "UINT16";
	      if (DFSDgetrange((void *) &(max.uint_16), (void *) &(min.uint_16))
		  != FAIL)
		printf("  **max: %u  min: %u\n\n",
		       max.uint_16, min.uint_16);
	      break;
	    case DFNT_INT32:
	      dt = "INT32";
	      if (DFSDgetrange((void *) &(max.int_32), (void *) &(min.int_32))
		  != FAIL)
		printf("  **max: %ld  min: %ld\n\n",
		       max.int_32, min.int_32);
	      break;
	    case DFNT_UINT32:
	      dt = "UINT32";
	      if (DFSDgetrange((void *) &(max.uint_32), (void *) &(min.uint_32))
		  != FAIL)
		printf("  **max: %lu  min: %lu\n\n",
		       max.uint_32, min.uint_32);
	      break;
	    case DFNT_UCHAR8:
	      dt = "UCHAR8";
	      if (DFSDgetrange((void *) &(max.char_8), (void *) &(min.char_8))
		  != FAIL)
		printf("  **max: %c  min: %c\n\n",
		       max.char_8, min.char_8);
	      break;
	    case DFNT_CHAR8:
	      dt = "CHAR8";
	      if (DFSDgetrange((void *) &(max.uchar_8), (void *) &(min.uchar_8))
		  != FAIL)
		printf("  **max: %c  min: %c\n\n",
		       max.uchar_8, min.uchar_8);
	      break;
	  }
	printf("  **Data Type: %s \n", dt);

       /*
        * * Display annotation for this data set
        */

	printf("\n* Annotation. (Refers to SDS #%d)\n\n", i);
	read_ann(hdf_file, DFTAG_NDG, reference, settings);
      }
}				/* END infoSDS */
